/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
/************************************************************
 * project=tweety                                           *
 * codename=kleiner eibnet/ip server                        *
 * version=1.1                                              *
 ************************************************************/

/**
 * @file receive_eib.c
 *
 * This file contains the code for the EIB receive thread.
 *
 * @date created 16.05.05,
 *       last modified 02.09.05
 *
 * @author bernhard erb
 *         bernhard.erb@gmx.net
 *
 */ 



/*
 * ********************************************* includes ***
 */

#include "common.h"

/*
 * ********************************************** globals ***
 */

int receive_eib_pipe = -1;

/*
 * ******************************************** functions ***
 */

/**
 * Closes the receive pipe.
 */
void ReceiveEib_freeRessources()
{				/* cleanup */
    if (receive_eib_pipe != -1) {
	close(receive_eib_pipe);
	receive_eib_pipe = -1;
    }
}

/**
 * This method acts as signal handler.
 * Covered signals:
 * SIGURS1: This signal is used to stop
 * the thread
 */
void ReceiveEib_HandleSignal(int sig)
{
    switch (sig) {
    case SIGUSR1:		/* terminate thread */
	ReceiveEib_freeRessources();	/* cleanup */
	pthread_exit(0);	/* exit with error code 0 */
	break;
    case SIGALRM:
      fprintf(stdout,"hoppla \n");
      break;

    default:
#ifdef DEBUG
	fprintf(stdout, "DEBUG: unknown signal\n");
#endif

	break;

    }
}

/**
 * This is the main method of the
 * receive eib thread. First all
 * needed ressources are allocated.
 * When a new message from bus arrives,
 * it is stored without revision in 
 * the receive pipe.
 */ 
void *receiveEib()
{
    byte *buffer = malloc(32);
    int size = 0;
 

    signal(SIGUSR1, ReceiveEib_HandleSignal);	/* SIGUSR1 for terminating */

    if ((receive_eib_pipe = open(SEND_PIPE, O_WRONLY)) == -1) {	/* open device */
      bailOut("Error: can't open send_pipe");
    }
   
   
    while (1) {
	if ((size = read(eib_fd, buffer, 32)) == -1) {	/* read the message from bus */
	  bailOut("Error: can't receive packet from eib ");
	}
	if (write(receive_eib_pipe, buffer, size) < 0) {	/* write it into pipe */
	    bailOut("Error: can't write packet to pipe");
	}

#ifdef DEBUG
	fprintf(stdout, "DEBUG: packet received from eib\n");
	zeig(buffer,size);
#endif

    }
}


/*
 * ************************************************ EOF ***
 */
