/*
 * Copyright (C) 2005 Bernhard Erb
 * Copyright (C) 2006 Martin Koegler <mkoegler@auto.tuwien.ac.at>

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/************************************************************
 * project=tweety                                           *
 * codename=kleiner eibnet/ip server                        *
 * version=1.1                                              *
 ************************************************************/

/**
 * @file send_eib.c
 *
 * This file contains the code for the EIB send thread.
 *
 * @date created 16.05.05,
 *       last modified 02.09.05
 *
 * @author bernhard erb
 *         bernhard.erb@gmx.net
 *
 */



/*
 * ********************************************* includes ***
 */


#include "common.h"


/*
 * ********************************************** globals ***
 */


int send_eib_p = -1;

/*
 * ******************************************* prototypes ***
 */
void bailOut(char *message);	/* forward declaration: method in header.c */


/*
 * ******************************************** functions ***
 */

/**
 * Closes the receive pipe
 */
void SendEib_freeRessources()
{				/* cleanup */
    if (send_eib_p != -1) {
	close(send_eib_p);
    }
}

/**
 * Signal handler for receive eib
 * thread. SIGUSR1 causes the thread
 * to finish
 */
void SendEib_signalHandler(int sig)
{
    switch (sig) {
    case SIGUSR1:		/* SIGUSR 1 is the term signal */
	SendEib_freeRessources();	/* cleanup */
	pthread_exit(0);	/* exit thread with 0 */
	break;
    default:			/* never  reached */
#ifdef DEBUG
	fprintf(stdout, "DEBUG: uknown signal\n");
#endif
	break;
    }
}

/**
 * Converts a cEMI packet into
 * emi message format. Note that
 * EIBnet/IP communicates with
 * cEMI packets, whereas BCU only
 * accepts EMI message format.
 */
byte *convert_CEMI_to_EMI(byte * buffer)
{				/* returns pointer to emi frame */
    CEMI_L_DATA_MESSAGE *cemi = (CEMI_L_DATA_MESSAGE *) buffer;
    EMI_L_DATA_MESSAGE *emi =
	(EMI_L_DATA_MESSAGE *) malloc(sizeof(EMI_L_DATA_MESSAGE) +
				      cemi->apci_length);

    switch (cemi->mc) {
    case 0x11:			/* L_Data_Req message */
	emi->mc = cemi->mc;	/* message code remains the same */
	emi->ctrl1 = cemi->ctrl1;
	emi->saddr = cemi->saddr;
	emi->daddr = cemi->daddr;
	emi->ctrl2 = cemi->apci_length | (cemi->ctrl2 & 0xF0);
	emi->npdu = cemi->apci;

	memcpy((byte *) emi + sizeof(EMI_L_DATA_MESSAGE), (byte *) cemi + sizeof(CEMI_L_DATA_MESSAGE), cemi->apci_length);	/* copy the message content in the emi frame */

	break;

    case 0x2E:			/* L_Data_Con not needed in KNoppiX so not implemented */
	break;

    case 0x29:			/* L_Data_Ind not needed in KNoppiX so not implemented */
	break;

    }

    return (byte *) emi;
}

/**
 * This method blocks until a new
 * message arrives in send pipe.
 * The message is converted into 
 * EMI format using convert_CEMI_to_EMI
 * method and sent to the bcu. 
 * As blocking write is used, the
 * write instruction returns after
 * successfull message processment
 */  
void *sendEib(void)
{
    CEMI_L_DATA_MESSAGE *cemi;
    EMI_L_DATA_MESSAGE *emi;
    int cemi_length;

    byte *buffer = (byte *) malloc(64);	/* allocate buffer */

    signal(SIGUSR1, SendEib_signalHandler);	/* Register SIGUSR1 for terminating */


    if ((send_eib_p = open(RECEIVE_PIPE, O_RDONLY)) == -1) {	/* open eib device */
	bailOut("Error: can't open pipe receive_pipe");
    }

    while (1) {

	if (read(send_eib_p, (char *) buffer, 10) < 0) {	/* read the first 10 bytes */
	    bailOut("Error: cannot read from receive_pipe");
	}


	cemi_length = ((CEMI_L_DATA_MESSAGE *) buffer)->apci_length;

	/* read up to cEMI length */

	if (read(send_eib_p, (char *) buffer + 10, cemi_length) < 0) {
	    bailOut("Error: can't read from receive_pipe");
	}

	cemi = (CEMI_L_DATA_MESSAGE *) buffer;

	/* convert cEMI to EMI */
	emi = (EMI_L_DATA_MESSAGE *) convert_CEMI_to_EMI(buffer);

	/* write message to bus */
	if (write
	    (eib_fd, emi,
	     sizeof(EMI_L_DATA_MESSAGE) + cemi->apci_length) == -1) {
	    bailOut("Error: can't write to bus");
	}

#ifdef DEBUG
	fprintf(stdout, "DEBUG: packet wrote to eib\n");
	zeig((byte *)emi, sizeof(EMI_L_DATA_MESSAGE) + cemi->apci_length);
#endif

	free(emi);		/* free the EMI frame */
    }
    free(buffer);		/* free buffer */

}


/*
 * ****************************************** EOF ***
 */
